@@ -95,6 +95,7 @@ class IconEntry(object):
         self._icon_win = None
         self._entry = entry
         self._tooltip = Tooltip(self)
+        self._locked = False
         entry.connect('enter-notify-event',
                       self._on_entry__enter_notify_event)
         entry.connect('leave-notify-event',
@@ -166,8 +167,7 @@ class IconEntry(object):
             else:
                 win.show()
 
-        # Hack: This triggers a .recompute() which is private
-        entry.set_visibility(entry.get_visibility())
+        self._recompute()
         entry.queue_draw()
 
     def construct(self):
@@ -211,6 +211,8 @@ class IconEntry(object):
             self._icon_win = None
 
     def update_background(self, color):
+        if self._locked:
+            return
         if not self._icon_win:
             return
 
@@ -218,6 +220,9 @@ class IconEntry(object):
 
         self.draw_pixbuf()
 
+    def get_background(self):
+        return self._entry.style.base[gtk.STATE_NORMAL]
+
     def resize_windows(self):
         if not self._pixbuf:
             return
@@ -237,6 +242,7 @@ class IconEntry(object):
             #        The text jumps without this
             textw -= 2
             self._text_area.move_resize(textx, texty, textw, texth)
+            self._recompute()
         elif self._pos == gtk.POS_RIGHT:
             self._text_area.resize(textw, texth)
             iconx += textw
@@ -278,3 +284,19 @@ class IconEntry(object):
             self._pos = gtk.POS_LEFT
         else:
             self._pos = gtk.POS_RIGHT
+
+    def _recompute(self):
+        # Protect against re-entrancy when inserting text, happens in DateEntry
+        if self._locked:
+            return
+
+        # Hack: This triggers a .recompute() which is private
+        self._locked = True
+        self._entry.insert_text('')
+        self._locked = False
+        # Another option would be to change the visibility:
+        #
+        # visibility = self._entry.get_visibility()
+        # self._entry.set_visibility(not visibility)
+        # self._entry.set_visibility(visibility)
+
